; ===============================
; Set 16-bit priveliged i386 mode
; ===============================

.386p
OPTION SEGMENT:USE16
OPTION casemap:none

BootSeg SEGMENT PARA PRIVATE
	ASSUME cs:BootSeg,ds:BootSeg,ss:BootSeg,es:BootSeg
	ORG 7C00h
	
	; =================================================
	; Make sure that this is compatible with all BIOSes
	; =================================================

Boot_Start:

	jmp Boot_StartContinue

Boot_StartContinue:

	; ==============
	; Setup segments
	; ==============
	
	; ----------------------
	; First, stop interrupts
	; ----------------------
	
	cli
	
	; ------------------------------------------------
	; Code is already correct, set the code using this
	; ------------------------------------------------
	
	mov ax, cs
	mov ds, ax
	mov es, ax
	
	; --------------------------------------------------------------
	; Put the stack at 90000h (remember that vidmem starts at A0000)
	; --------------------------------------------------------------

	mov	ax, 09000h
	mov	ss, ax
	mov	sp, 0F800h
	
	; -------------------
	; Get interrupts back
	; -------------------
	
	sti
	
	; ===============================================
	; Check string operations are going the right way
	; ===============================================
	
	cld
	
	; =======================================================================
	; Initialise the screen (640 x 200 pixels, 80 x 25 characters, Text only)
	; =======================================================================
	
	mov ax, 0003h
	int 10h
	
	; =====================
	; Print welcome message
	; =====================
	
	mov  si, offset szLoadPers
	call Boot_PrntStrng
	
	; ==========
	; Reset disk
	; ==========

	mov ah, 00h
	mov dl, 00h
	int 13h

	; ============
	; Load sectors
	; ============
	
	; ----------------------
	; Setup 3 times try loop
	; ----------------------
	
	xor ax, ax
	
Boot_ReadAttempt:

	cmp ax, 3
	jnb Boot_DiskFail
	
	inc ax
	push ax
	
	; ----------------
	; Read the sectors
	; ----------------

	mov ah, 02h				; AH = 2 (read from disk mode)
	mov al, bNumSecs		; AL = bNumSecs (number of sectors to read)
	mov cl, bSector			; CL = Start sector
	mov ch, bCylinder		; CH = Cylinder
	mov dh, bHead			; DH = Head
	mov dl, bDrive			; DL = Drive
	mov bx, wNxtSecAddr		; BX = Buffer address
	int 13h					; Execute disk read
	
	; ----
	; Loop
	; ----
	
	pop ax
	
	jc  Boot_ReadAttempt
	
	; =======
	; Read OK
	; =======
	
	mov  si, offset szFDDRead
	call Boot_PrntStrng
		
	; =======================================
	; Continue to the next bit of (real) code
	; =======================================

Boot_End:
	DB 0EAh
	DW 07E00h 
	DW 00000h
	
	; =========================================================
	; Restart the computer: something must have gone very wrong
	; =========================================================
	
Boot_DiskFail:

	mov  si, offset szFDDError
	call Boot_PrntStrng
	
	DB 0EAh
	DW 00000h
	DW 0FFFFh
	
	; #################
	; Boot data section
	; #################
	
Boot_Data:

	szLoadPers	db "Ossa's Bootloader version 0.0.0.0", 0Dh, 0Ah, 0
	szFDDRead	db "Read from FDD OK...", 0Dh, 0Ah, "Bootload complete: Starting Perspective", 0Dh, 0Ah, 0
	szFDDError	db "FDD read error...", 0Dh, 0Ah, "Press any key to restart...", 0Dh, 0Ah, 0
	
	bNumSecs	db 0Ah
	bHead		db 00h
	bCylinder	db 00h
	bDrive		db 00h
	bSector		db 02h
	wNxtSecAddr	dw 7E00h
	
	; ######################
	; Boot utility functions
	; ######################
	
	; ====================================================
	; Prints a string to the screen (si = message address)
	; ====================================================
	
Boot_PrntStrng:
	
	; -----------------------------------------------------------------------------------------------------
	;
	; VIDEO - TELETYPE OUTPUT (Int 10h, AH = 0Eh)
	;
	; AH = 0Eh										[Set]
	; AL = character to write						[Set]
	; BH = page number								[Ignore]
	; BL = foreground color (graphics modes only)	[Ignore]
	;
	; Return: Nothing
	; Desc: Display a character on the screen, advancing the cursor and scrolling the screen as necessary
	; BUG: If the write causes the screen to scroll, BP is destroyed by BIOSes for which AH=06h destroys BP
	;      [Ignore as BP unused]
	;
	; -----------------------------------------------------------------------------------------------------
	
	; ----------------------------------
	; Get current char to al and do si++
	; ----------------------------------
	
	lodsb
	
	; -------------------------
	; Check for NULL terminator
	; -------------------------
	
	or	al,al
	jz	Boot_PrntStrng_Ret

	; -------
	; Save si
	; -------

	push si
	
	; ----------
	; Print char
	; ----------
	
	mov	ah, 0Eh
	int	10h
	
	; ----------
	; Restore si
	; ----------
	
	pop	si
	
	; ----
	; Loop
	; ----
	
	jmp	Boot_PrntStrng
	
	; -------------
	; Exit function
	; -------------
	
Boot_PrntStrng_Ret:
	ret

	; ====================================================================
	; The boot signature: make the BIOS know that we are a bootable sector
	; ====================================================================
	
BootSignature:
	ORG	Boot_Start + 510
	DW  0AA55h

BootSeg ENDS

end
